{%extends "base.html"%}
{% block content%}
<p>The package must be exact:</p>
<form action='/find-packages' method="POST">
  <input type='text' name='search_box'/> <input type='submit'>
</form>

{%if releases%}
<h2>Click on release to download</h2>
<ul>
  {% for r in releases %}
  <li>
    <a href='/package/{{search_term}}/{{r}}'>{{r}}</a>
  </li>
  {% endfor %}
</ul>
{%endif%}
{% endblock %}
