{%extends "base.html"%}
{% block content%}
<a href='/index'>See index</a>
<p>There are two ways to mirror packages from pypi. 
	<ol>
		<li>The first is to use a requirements file in the same format that pip freeze produces.</li>
		<li>The second is to use the search functionality and choose a package</li>
	</ol>
</p>
<ul>
	<li><a href='/load-requirements'>Upload requirements file</a></li>
	<li><a href='/find-packages'>Search Pypi</a></li>
</ul>
{% endblock %}
