
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd"> 

<html xmlns="http://www.w3.org/1999/xhtml"> 
<head> 
<meta http-equiv="Content-Type" content="text/html; charset=utf-8" /> 

<title>test</title> 
<link rel="stylesheet" href="/static/style.css" type="text/css" /> 

</head> 
<body> 
<div class=indexwrapper> 
	<div class="document"> 
		<div class="documentwrapper"> 
			<div class="bodywrapper"> 
				<div class="body"> 
					<div class="section" id="flask-themes"> 
						<div class="topic">test</div>
						<h1>Flask-Themes<a class="headerlink" href="#flask-themes" title="Permalink to this headline">¶</a></h1> 
						<p>Flask-Themes makes it easy for your application to support a wide range of
							appearances.</p>
					</div> 
				</div>
			</div> 
		</div> 
	</div> 
</div> 
<div class="clearer"></div> 
</body> 
</html>