"""
CheesePrism
===========

 - wsgiapp: main wsgi entry point

 - request: custom request object

 - pipext: utilities for parsing and downloading via requirements
           files

 - index: main code for managing file based python index, includes
          package management event subscribers

 - resources: pyramid in memory resource tree

 - event: package management events

 - utils: helpers, etc

 - desc: descriptors

"""


    

