{%extends "base.html"%}
{%block content%}
<p>Upload a requirements file to populate the index</p>
<form method="POST" enctype="multipart/form-data" action="/load-requirements"> 
  File to upload: <input type="file" name="req_file"><br> 
  <br /> 
  <input type="submit" value="Press"> to upload the file!
</form>
{%endblock%}
