<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
          "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd"> 

<html xmlns="http://www.w3.org/1999/xhtml"> 
  <head> 
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8" /> 

    <title>
      {% block title %}
      {{ title }}
      {% endblock %}
    </title> 
    {%if csspath%}
    <link rel="stylesheet" href="{{csspath}}" type="text/css" /> 
    {%endif%}
    {%block js%}
    {%endblock%}
  </head> 
  <body> 
    <div class=indexwrapper> 
      <div class="document"> 
	<div class="documentwrapper"> 
	  <div class="bodywrapper"> 
	    <div class="body"> 
	      <div class="section"> 
		{%block content%}
		{%endblock%}
	      </div> 
	    </div>
	  </div> 
	</div> 
      </div> 
    </div> 
    <div class="clearer"></div> 
  </body> 
</html>
